local PetShopping = LibStub("AceAddon-3.0"):GetAddon("PetShopping")
local PJ = PetShopping:NewModule("PetJournal")
local LibPetJournal = LibStub('LibPetJournal-2.0')

PJ._missing_species = PJ._missing_species or {}
PJ._known_species = PJ._known_species or {}
PJ._source_map = PJ._source_map or {}

function PJ:IterateMissingSpecies()  return ipairs(self._missing_species) end
function PJ:IterateAllSpecies()  return ipairs(self._all_species) end
function PJ:GetPetType(idx)
    local t = {}
    t[1] = "Humanoid"
    t[2] = "Dragonkin"
    t[3] = "Flying"
    t[4] = "Undead"
    t[5] = "Critter"
    t[6] = "Magic"
    t[7] = "Elemental"
    t[8] = "Beast"
    t[9] = "Aquatic"
    t[10] = "Mechanical"
    return t[idx]
end
function PJ:GetPetSource(pet) return self._source_map[pet]; end
function PJ:GetAchieve(pet)
    if (pet <= 1235 and pet >= 1226) then return "Raiding with Leashes II" end
    if (pet <= 1156 and pet >= 1143) then return "Raiding with Leashes" end
    if (pet <= 1634 and pet >= 1622) then return "Raiding with Leashes III" end
    if ((pet <= 1968 and pet >= 1954) or (pet == 1952)) then return "Raiding with Leashes IV" end
end

function PJ:GetSourceFromTooltip(tt)
    local s = tt:match("^|c%w%w%w%w%w%w%w%w(.-):")
    --can't figure out how to match in-game shop
    if s == nil then s = "In-Game Shop" end
    return s
end

function PJ:Update()

    PJ._missing_species = {}
    PJ._known_species = {}
    PJ._all_species = {}
    PJ._source_map = {}

    for i,petid in LibPetJournal:IteratePetIDs() do
        local speciesID = C_PetJournal.GetPetInfoByPetID(petid)
        self._known_species[speciesID] = true
    end

    for i,speciesID in LibPetJournal:IterateSpeciesIds() do

        local name, icon, petType, creatureID, tooltipSource, _, _, _, tradable, _, obtainable = C_PetJournal.GetPetInfoBySpeciesID(speciesID);
        if (tradable and obtainable) then
            if not self:IsSpeciesKnown(speciesID) then
                tinsert(PJ._missing_species, {
                    species_id = speciesID,
                    creature_id = creatureID,
                    pet_type = petType,
                    name = name,
                    tradable = tradable
                })
            end
            tinsert(PJ._all_species, {
                species_id = speciesID,
                creature_id = creatureID,
                pet_type = petType,
                name = name,
                tradable = tradable
            })
            PJ._source_map[speciesID] = PJ:GetSourceFromTooltip(tooltipSource)
        end
    end
end

function PJ:IsSpeciesKnown(species_id)
    if not self._known_species[species_id] then
        return false
    end
    return true
end

LibPetJournal.RegisterCallback(PJ, "PetListUpdated", "Update")

