---------------
-- LIBRARIES --
---------------
local addonName, addonTable = ...;
local utils = addonTable.utils;
local _

local AceAddon = LibStub("AceAddon-3.0");

PetShopping = AceAddon:NewAddon(addonName, "AceConsole-3.0", "AceEvent-3.0");
local Listener = CreateFrame('Frame', nil, UIParent);

---------------
-- DB --
---------------
local defaults = {
    global = {
        enabled = true,
        filters = {},
        families = {
            Humanoid = true,
            Dragonkin = true,
            Flying = true,
            Undead = true,
            Critter = true,
            Magic = true,
            Elemental = true,
            Beast = true,
            Aquatic = true,
            Mechanical = true
        },
        sources = {},
        achieves = {}
    }
}
defaults.global.sources["Drop"] = true
defaults.global.sources["Quest"] = true
defaults.global.sources["Vendor"] = true
defaults.global.sources["Profession"] = true
defaults.global.sources["Pet Battle"] = true
defaults.global.sources["Achievement"] = true
defaults.global.sources["World Event"] = true
defaults.global.sources["Promotion"] = true
defaults.global.sources["Trading Card Game"] = true
defaults.global.sources["In-Game Shop"] = true
defaults.global.sources["Discovery"] = true


defaults.global.achieves["Raiding with Leashes"] = false
defaults.global.achieves["Raiding with Leashes II"] = false
defaults.global.achieves["Raiding with Leashes III"] = false
defaults.global.achieves["Raiding with Leashes IV"] = false
---------------
-- LIST --
---------------

--check into tsm shopping list
--make list into links instead of text

function explode(div,str)
    if (div=='') then return false end
    local pos,arr = 0,{}
    for st,sp in function() return string.find(str,div,pos,true) end do
        table.insert(arr,string.sub(str,pos,st-1))
        pos = sp + 1
    end
    table.insert(arr,string.sub(str,pos))
    return arr
end

function PetShopping:SetMsg(msg)
    PS_Message_Text:SetText(msg)
end

function PetShopping:SendToAuctionator()

    local str = PS_List_EditBox:GetText();
--    local plist = explode("\n",str)
    local listname = PS_ListName_EditBox:GetText();
    if (str == "" or str == nil) then PetShopping:SetMsg("Empty list") return end
    if (listname == "" or listname == nil) then PetShopping:SetMsg("Requires Shopping List name") return end

    if ( AuctionatorLoaded ) then
        local slist = Atr_SList.FindByName(listname, nil);
        if ( not slist ) then
            slist = Atr_SList.create(listname, false, true);
        end
        slist:Clear();

        for _, pet in ipairs(explode("\n",str)) do
            slist:AddItem(pet);
        end
        table.insert (AUCTIONATOR_SHOPPING_LISTS, slist)
        table.sort (AUCTIONATOR_SHOPPING_LISTS, Atr_SortSlists);
        PetShopping:SetMsg("Shopping list sent.")
    else
        PetShopping:SetMsg("Requires Auctionator")
    end
end

-----------------------------------------

function PetShopping:PS_ShowShoppingList()

    local plist = PetShopping:GetList()
    local text;
    local sortkey, info;
    local editBox = PS_List_EditBox;

    table.sort (plist, function(x, y) return x < y; end )

    editBox:ClearFocus()
    editBox:SetText(utils.ListToText(plist))
    editBox:SetCursorPosition(0)
    PS_ListName_EditBox:SetBackdropColor(red);
end

---------------
-- CONFIG --
---------------

function PetShopping:PS_ShowOptionTooltip (elem)

    local name = elem:GetName();
    local text;

    if (utils.StringContains (name, "Missing")) then
        text = "If this option is checked, the shopping list will only contain missing species."
    end
    if (utils.StringContains (name, "Families")) then
        text = "The shopping list will contain the families selected on this dropdown."
    end
    if (utils.StringContains (name, "Auctionator")) then
        text = "Create an Auctionator Shopping List from this list."
    end
    if (utils.StringContains (name, "Achieve")) then
        text = "The shopping list will contain ONLY species from the checked achievements."
    end
    if (utils.StringContains (name, "Show")) then
        text = "Create and display the shopping list"
    end

    if (text) then
        local titleFrame = _G[name.."_CB_Text"] or _G[name.."_Text"] or _G[name];

        local titleText = titleFrame and titleFrame:GetText() or "???";

        GameTooltip:SetOwner(elem, "ANCHOR_LEFT");
        GameTooltip:SetText(titleText, 0.9, 1.0, 1.0);
        GameTooltip:AddLine(text, 0.5, 0.5, 1.0, 1);
        GameTooltip:Show();
    end

end


function PetShopping:PS_Option_OnClick (elem)
    PetShopping.db.global.filters[elem] = not PetShopping.db.global.filters[elem];
end

function PetShopping:PS_OptionFrame_Init ()
    local filters = PetShopping.db.global.filters

    filters.PS_MissingOpt_CB = filters.PS_MissingOpt_CB or false;
    PS_MissingOpt_CB:SetChecked(filters.PS_MissingOpt_CB);

end

function PetShopping:OnCursorChanged(_,y,_,h)
    self.cursorOffset = y
    self.cursorHeight = h
    self.handleCursorChange = true;
    self:SetScript("OnUpdate",PetShopping.OnCursorOnUpdate)
end

function PetShopping:OnCursorOnUpdate(elapsed)
    self:SetScript("OnUpdate",nil)
    ScrollingEdit_OnUpdate(self,elapsed,self:GetParent())
end

function PS_Families_DD_Option_OnClick(self)
    PetShopping.db.global.families[self:GetText()] = not PetShopping.db.global.families[self:GetText()]
end

function PS_Sources_DD_Option_OnClick(self)
    PetShopping.db.global.sources[self:GetText()] = not PetShopping.db.global.sources[self:GetText()]
end

function PS_Achieves_DD_Option_OnClick(self)
    PetShopping.db.global.achieves[self:GetText()] = not PetShopping.db.global.achieves[self:GetText()]
end

function PS_InitializeFamilies(self, level)

    if (PetShopping.db) then
        local info
        local families = PetShopping.db.global.families

        for kind, chk in pairs(families) do
            info = UIDropDownMenu_CreateInfo()
            info.text = kind
            info.keepShownOnClick = true
            info.checked = chk
            info.func = PS_Families_DD_Option_OnClick
            UIDropDownMenu_AddButton(info)

        end
    end

end

function PS_InitializeSources(self, level)

    if (PetShopping.db) then
        local info
        local sources = PetShopping.db.global.sources

        for kind, chk in pairs(sources) do
            info = UIDropDownMenu_CreateInfo()
            info.text = kind
            info.keepShownOnClick = true
            info.checked = chk
            info.func = PS_Sources_DD_Option_OnClick
            UIDropDownMenu_AddButton(info)

        end
    end

end

function PS_InitializeAchieves(self, level)

    if (PetShopping.db) then
        local info
        local achieves = PetShopping.db.global.achieves;

        local tkeys = {}
        for k in pairs(achieves) do table.insert(tkeys, k) end
        table.sort(tkeys)


        for _, kind in pairs(tkeys) do
            local chk = achieves[kind]
            info = UIDropDownMenu_CreateInfo()
            info.text = kind
            info.keepShownOnClick = true
            info.checked = chk
            info.func = PS_Achieves_DD_Option_OnClick
            UIDropDownMenu_AddButton(info)

        end
    end

end

function PetShopping:Families_DD_OnLoad(self)  UIDropDownMenu_Initialize(self, PS_InitializeFamilies) end
function PetShopping:Sources_DD_OnLoad(self) UIDropDownMenu_Initialize(self, PS_InitializeSources) end
function PetShopping:Achieves_DD_OnLoad(self) UIDropDownMenu_Initialize(self, PS_InitializeAchieves) end

function PetShopping:DropDown_ButtonOnClick(self, button, down)
    local name = self:GetName()

    if (utils.StringContains (name, "Families")) then
        ToggleDropDownMenu(1, nil, PS_Families_DD_Option, name, 0,0)
    end
    if (utils.StringContains (name, "Sources")) then
        ToggleDropDownMenu(1, nil, PS_Sources_DD_Option, name, 0,0)
    end
    if (utils.StringContains (name, "Achieves")) then
        ToggleDropDownMenu(1, nil, PS_Achieves_DD_Option, name, 0,0)
    end
end

function PetShopping:GetList()
    local PJ =  PetShopping:GetModule("PetJournal")
    local filters = PetShopping.db.global.filters
    local families = PetShopping.db.global.families
    local sources = PetShopping.db.global.sources
    local achieves = PetShopping.db.global.achieves
    local slist = {}
    local achieves_chk = false
    for _,j in pairs(achieves) do if j then achieves_chk = true end end

    if (filters.PS_MissingOpt_CB) then
        for _, pet in PJ:IterateMissingSpecies() do
            if (families[PJ:GetPetType(pet.pet_type)]
                    and sources[PJ:GetPetSource(pet.species_id)]
                    and (not PS_Achieves_Checked or achieves[PJ:GetAchieve(pet.species_id)])) then
                tinsert(slist, pet.name)
            end
        end
    else
        for _, pet in PJ:IterateAllSpecies() do
            if (families[PJ:GetPetType(pet.pet_type)]
                    and sources[PJ:GetPetSource(pet.species_id)]
                    and (not achieves_chk or achieves[PJ:GetAchieve(pet.species_id)])) then
                tinsert(slist, pet.name)
            end
        end
    end
    return slist;
end
---------------
-- CORE --
---------------

function PetShopping:OnInitialize()
    -- setup db
    self.db = LibStub("AceDB-3.0"):New("PetShoppingDB", defaults, true);

    -- setup chat commands
    self:RegisterChatCommand("petshop", "SlashHandler")

    -- setup menu
    --self:RegisterMenu();
    PetShopping:PS_OptionFrame_Init();
    --UIDropDownMenu_Initialize(self, PS_InitializeSpecies)
end

function PetShopping:OnEnable()

    Listener:SetScript('OnEvent', function(_, event) if PetShopping[event] then PetShopping[event](PetShopping) end end)
    PetShopping.db.global.enabled = true;

end

function PetShopping:OnDisable()
    self:UnregisterAllEvents();

    for fontString, _ in pairs(animating) do
        recycleFontString(fontString);
    end

    PetShopping.db.global.enabled = false;
end

function PetShopping:Toggle()
    PS_Frame:SetShown(not PS_Frame:IsVisible())
end

function PetShopping:SlashHandler(msg)
    local command = msg:match("^(%S*)%s*(.-)$");

    if (command=="list") then PetShopping:MissingList()
    else PetShopping:Toggle()
    end

end